package com.ruoyi.project.system.mapper;

import com.ruoyi.project.system.domain.ProductCraftParams;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 工艺选择项Mapper接口
 *
 * @author zhu
 * @date 2023-10-26
 */
public interface ProductCraftParamsMapper {
    /**
     * 查询工艺选择项
     *
     * @param id 工艺选择项主键
     * @return 工艺选择项
     */
    public ProductCraftParams selectProductCraftParamsById(Long id);

    /**
     * 查询工艺选择项列表
     *
     * @param productCraftParams 工艺选择项
     * @return 工艺选择项集合
     */
    public List<ProductCraftParams> selectProductCraftParamsList(ProductCraftParams productCraftParams);

    /**
     * 新增工艺选择项
     *
     * @param productCraftParams 工艺选择项
     * @return 结果
     */
    public int insertProductCraftParams(ProductCraftParams productCraftParams);

    /**
     * 修改工艺选择项
     *
     * @param productCraftParams 工艺选择项
     * @return 结果
     */
    public int updateProductCraftParams(ProductCraftParams productCraftParams);

    /**
     * 删除工艺选择项
     *
     * @param id 工艺选择项主键
     * @return 结果
     */
    public int deleteProductCraftParamsById(Long id);

    /**
     * 批量删除工艺选择项
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteProductCraftParamsByIds(Long[] ids);

    /**
     * 删除数据
     *
     * @param productId
     * @param materialId
     * @return
     */
    public int deleteByProductId(Long productId, Long materialId);

    /**
     * ids获取list
     *
     * @param ids
     * @return
     */
    public List<ProductCraftParams> selectProductCraftParamsByIds(@Param("ids") List<Long> ids);

}
